"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _reactReconciler = require("react-reconciler");

var _reactReconciler2 = _interopRequireDefault(_reactReconciler);

var _emptyObject = require("fbjs/lib/emptyObject");

var _emptyObject2 = _interopRequireDefault(_emptyObject);

var _createElement = require("./createElement");

var _diffProperties = require("../util/diffProperties");

var _diffProperties2 = _interopRequireDefault(_diffProperties);

var _dmx = require("../util/dmx");

var _dmx2 = _interopRequireDefault(_dmx);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ReconcilerConfig = {
  appendInitialChild: function appendInitialChild(parentInstance, child) {
    if (parentInstance.appendChild) {
      parentInstance.appendChild(child);
    }
  },
  createInstance: function createInstance(type, props, internalInstanceHandle) {
    return (0, _createElement.createElement)(type, props, internalInstanceHandle);
  },
  createTextInstance: function createTextInstance(text, rootContainerInstance, internalInstanceHandle) {
    return text;
  },
  finalizeInitialChildren: function finalizeInitialChildren(wordElement, type, props) {
    return false;
  },
  getPublicInstance: function getPublicInstance(inst) {
    return inst;
  },
  prepareForCommit: function prepareForCommit() {
    // noop
  },
  prepareUpdate: function prepareUpdate(wordElement, type, oldProps, newProps) {
    return (0, _diffProperties2.default)(type, oldProps, newProps);
  },
  resetAfterCommit: function resetAfterCommit() {
    // noop
  },
  resetTextContent: function resetTextContent(wordElement) {
    // noop
  },
  getRootHostContext: function getRootHostContext(rootInstance) {
    // You can use this 'rootInstance' to pass data from the roots.
  },
  getChildHostContext: function getChildHostContext() {
    return _emptyObject2.default;
  },
  shouldSetTextContent: function shouldSetTextContent(type, props) {
    return false;
  },


  now: function now() {
    return Date.now();
  },

  useSyncScheduling: true,

  mutation: {
    appendChild: function appendChild(parentInstance, child) {
      if (parentInstance.appendChild) {
        parentInstance.appendChild(child);
      }
    },
    appendChildToContainer: function appendChildToContainer(parentInstance, child) {
      if (parentInstance.appendChild) {
        parentInstance.appendChild(child);
      }
    },
    removeChild: function removeChild(parentInstance, child) {
      _dmx2.default.update(child.root, (0, _defineProperty3.default)({}, child.props.channel, 0));
      parentInstance.removeChild(child);
    },
    removeChildFromContainer: function removeChildFromContainer(parentInstance, child) {
      if (child.universe) {
        // Clear out all of the channels first.
        child.universe.updateAll(0);
        // Give it some time to write the 0s to the device
        setTimeout(function () {
          child.universe.close();
          child.universe = null;
          delete child.universe;
        }, 50);
      }
      parentInstance.removeChild(child);
    },
    insertBefore: function insertBefore(parentInstance, child, beforeChild) {
      //Order doesn't matter. Just throw it in.
      if (parentInstance.appendChild) {
        parentInstance.appendChild(child);
      }
    },
    commitUpdate: function commitUpdate(instance, updatePayload, type, oldProps, newProps) {
      instance.props = newProps;
      _dmx2.default.update(instance.root, instance.render());
    },
    commitMount: function commitMount(instance, updatePayload, type, oldProps, newProps) {
      // noop
    },
    commitTextUpdate: function commitTextUpdate(textInstance, oldText, newText) {
      textInstance.children = newText;
    }
  }
};

var ReactDMX = (0, _reactReconciler2.default)(ReconcilerConfig);

exports.default = ReactDMX;